package provider

import scala.concurrent.Future
import concurrent.ExecutionContext.Implicits.global

class PlaylistResolverSync(
    musicProvider: MusicProviderSync,
    playlistProvider: PlaylistProviderSync[MusicKey]
) extends PlaylistProviderSync[MusicEntry]:
  def retrieveSync(key: PlaylistKey): Option[List[MusicEntry]] =
    val playlist = playlistProvider.retrieveSync(key)
    playlist.map {
      _.map { key =>
        musicProvider.retrieveSync(key)
      }.flatten
    }
  def retrieveKeysSync: Set[PlaylistKey] =
    playlistProvider.retrieveKeysSync

class PlaylistResolver(
    musicProvider: MusicProvider,
    playlistProvider: PlaylistProvider[MusicKey]
) extends PlaylistProvider[MusicEntry]:
  def retrieve(key: PlaylistKey): Future[List[MusicEntry]] =
    for
      musicKeys <- playlistProvider.retrieve(key)
      musicEntries <- Future.traverse(musicKeys)(musicProvider.retrieve)
    yield musicEntries

    // Makes the last test fail:
    // import scala.concurrent.Await
    // import scala.concurrent.duration.Duration
    // Future:
    //   for musicKey <- Await.result(playlistProvider.retrieve(key), Duration.Inf)
    //   yield Await.result(musicProvider.retrieve(musicKey), Duration.Inf)

  def retrieveKeys: Future[Set[PlaylistKey]] =
    playlistProvider.retrieveKeys
