package provider

import ProviderUtils.*
import concurrent.ExecutionContext.Implicits.global
import scala.concurrent.Future
import scala.util.Success

class RacingProviderSync[Key, Entry](
    val providerEU: ProviderSync[Key, Entry],
    val providerUS: ProviderSync[Key, Entry]
) extends ProviderSync[Key, Entry]:

  def retrieveSync(key: Key): Option[Entry] =
    providerEU.retrieveSync(key) match
      case Some(value) => Some(value)
      case None        => providerUS.retrieveSync(key)

  def retrieveKeysSync: Set[Key] =
    providerEU.retrieveKeysSync ++ providerUS.retrieveKeysSync

class RacingProvider[Key, Entry](
    val providerEU: Provider[Key, Entry],
    val providerUS: Provider[Key, Entry]
) extends Provider[Key, Entry]:

  def retrieve(key: Key): Future[Entry] =
    firstSuccessOf(providerEU.retrieve(key), providerUS.retrieve(key))

    // Makes the "should query both databases before waiting for results" test fail:
    // providerEU.retrieve(key).recoverWith(_ => providerUS.retrieve(key))

  def retrieveKeys: Future[Set[Key]] =
    providerEU.retrieveKeys.zip(providerUS.retrieveKeys).map(_ ++ _)

    // Makes the last test fail:
    // for keysEu <- providerEU.retrieveKeys; keysUs <- providerUS.retrieveKeys yield keysEu ++ keysUs
