package provider

import TestDb.*

import scala.concurrent.{Await, Future, Promise}
import concurrent.duration.DurationInt
import scala.language.adhocExtensions

// 10pts
class PlaylistResolverTest extends munit.FunSuite:
  val p =
    PlaylistResolver(RacingProvider(musicProviderEU, musicProviderUS), playlistProvider)

  test("retrieveKeys returns the keys of all playlists (2pts)"):
    val r = Await.result(p.retrieveKeys, 1.second)
    assertEquals(r, Set(PlaylistKey("1"), PlaylistKey("2"), PlaylistKey("3")))

  test("resolve playlist with songs on EU server should resolve them correctly (2pts)"):
    val r = Await.result(p.retrieve(PlaylistKey("1")), 1.second)
    val expected = List(
      dbContentEU(MusicKey("4")),
      dbContentEU(MusicKey("5")),
      dbContentEU(MusicKey("6"))
    )
    assertEquals(r, expected)

  test("resolve playlist with songs on US server should resolve them correctly (2pts)"):
    val r = Await.result(p.retrieve(PlaylistKey("2")), 1.second)
    val expected = List(
      dbContentUS(MusicKey("14")),
      dbContentUS(MusicKey("15")),
      dbContentUS(MusicKey("16"))
    )
    assertEquals(r, expected)

  test(
    "resolve playlist with songs on both EU and US server should resolve them correctly (2pts)"
  ):
    val r = Await.result(p.retrieve(PlaylistKey("3")), 1.second)
    val expected =
      List(
        dbContentEU(MusicKey("1")),
        dbContentEU(MusicKey("2")),
        dbContentEU(MusicKey("3")),
        dbContentEU(MusicKey("4")),
        dbContentEU(MusicKey("5")),
        dbContentEU(MusicKey("6")),
        dbContentEU(MusicKey("7")),
        dbContentEU(MusicKey("21")),
        dbContentEU(MusicKey("22")),
        dbContentEU(MusicKey("23")),
        dbContentUS(MusicKey("24")),
        dbContentEU(MusicKey("25"))
      )
    assertEquals(r, expected)

  test("songs should be retrieved in parallel (2pts)"):
    import concurrent.ExecutionContext.Implicits.global
    val fifteenQueried = Promise[Unit]()
    val fourteenQueried = Promise[Unit]()
    val mockUsDb = new Database(dbContentUS):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        if key == MusicKey("14") then
          fourteenQueried.success(())
          fifteenQueried.future.flatMap(_ => super.retrieve(key))
        else if key == MusicKey("15") then
          fifteenQueried.success(())
          super.retrieve(key)
        else if key == MusicKey("16") then
          fourteenQueried.future.flatMap(_ => super.retrieve(key))
        else
          super.retrieve(key)

    val p = PlaylistResolver(mockUsDb, playlistProvider)
    Await.result(p.retrieve(PlaylistKey("2")), 1.second)
