package provider

import TestDb.*

import scala.concurrent.{Await, Future, Promise}
import concurrent.duration.DurationInt
import scala.language.adhocExtensions

// 15pts
class SearchProviderTest extends munit.FunSuite:
  val p = SearchProvider(RacingProvider(musicProviderEU, musicProviderUS))

  test("query with no criteria returns all songs from both servers (1pt)"):
    val r =
      Await.result(p.retrieve(MusicSearchQuery(None, None, None, None)), 1.second)
    assertEquals(r, dbContentEU.values.toSet ++ dbContentUS.values.toSet)

  test("query with artist criteria returns the corresponding songs (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(Some("Dua Lipa"), None, None, None)),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentEU(MusicKey("8")),
        dbContentEU(MusicKey("9")),
        dbContentUS(MusicKey("24"))
      )
    )

  test("query with album criteria returns all songs from both servers (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(None, Some("Future Nostalgia"), None, None)),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentEU(MusicKey("8")),
        dbContentEU(MusicKey("9")),
        dbContentUS(MusicKey("24"))
      )
    )

  test("query with album and artist criteria returns the corresponding songs (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(
        Some("Dua Lipa"),
        Some("Future Nostalgia"),
        Some("Levitating"),
        None
      )),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentUS(MusicKey("24"))
      )
    )

  test("query with title criteria returns the corresponding songs (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(None, None, Some("Levitating"), None)),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentUS(MusicKey("24")),
        dbContentEU(MusicKey("25"))
      )
    )

  test("query with year criteria returns the corresponding songs (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(None, None, None, Some(2020))),
      1.second
    )
    assertEquals(
      r,
      (dbContentEU.values.toSet ++ dbContentUS.values.toSet).filter(
        _.year == 2020
      )
    )

  test("query with all criteria returns the corresponding songs (2pts)"):
    val r = Await.result(
      p.retrieve(MusicSearchQuery(
        Some("Dua Lipa"),
        Some("Future Nostalgia"),
        Some("Levitating"),
        Some(2020)
      )),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentUS(MusicKey("24"))
      )
    )

  test("query should not wait on keys of both providers before retrieving songs (2pts)"):
    import concurrent.ExecutionContext.Implicits.global

    val retrieveKeysUsCalled = Promise[Unit]()

    val mockMusicProviderEU = new Database(dbContentEU):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        retrieveKeysUsCalled.trySuccess(())
        super.retrieve(key)

    val mockMusicProviderUS = new Database(dbContentUS):
      override def retrieveKeys: Future[Set[MusicKey]] =
        retrieveKeysUsCalled.future.flatMap(_ => super.retrieveKeys)

    val p = SearchProvider(RacingProvider(mockMusicProviderEU, mockMusicProviderUS))
    val r = Await.result(
      p.retrieve(MusicSearchQuery(None, Some("Future Nostalgia"), None, None)),
      1.second
    )
    assertEquals(
      r,
      Set(
        dbContentEU(MusicKey("8")),
        dbContentEU(MusicKey("9")),
        dbContentUS(MusicKey("24"))
      )
    )
